/* *******************************************************
 * This code created and or modified by Chris Lasater
 * http://www.geocities.com/lasaterconsult
 * Please refer all questions or comments to chris.lasater@gmail.com
 * 
 * You can use this code however you like, but please, make sure you give me 
 * credit on any publishings you do from this code. 
 * 
 * No warrenty is implied for any code here.
 * 
 *  ******************************************************* 
 */


using System;
using System.Collections;
using System.IO;
using System.Reflection;

using Properties.Entity;
using Properties.Configuration;
using Properties.ImplClasses;

namespace Properties
{
	#region AOMFactory class
	/// <summary>
	/// Summary description for AOMFactory.
	/// </summary>
	public class AOMFactory
	{
		#region Private Variables
		private static string _directoryPath = string.Empty;
		private static string _fileName = string.Empty;
		private static Hashtable _entities;
		private static Hashtable _entityTypes;
		private static Hashtable _entityAttributes;
		#endregion
		
		#region Parameters
		/// <summary>
		/// Path with out filename
		/// </summary>
		/// <remarks>None</remarks>
		/// <value>string</value>
		protected static string DirectoryPath
		{
			get{return _directoryPath;}
			set{_directoryPath = value;}
		}
		/// <summary>
		/// File name only
		/// </summary>
		/// <remarks>None</remarks>
		/// <value>string</value>
		protected static string FileName
		{
			get{return _fileName;}
			set{_fileName = value;}
		}
		#endregion

		#region Public Factory Methods
		
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="name"></param>
		/// <returns></returns>
		public static JobEntity GetEntity(string name)
		{
		
			if(_entities == null) 
			{
				try
				{
					LoadData();
				}
				catch(Exception ex)
				{
					string message = ex.Message;
					//TODO : logging
				
				}
				if(_entities == null) 
					throw new FactoryException("No Configuration data found. AOMFactory : GetEntity");
				//set up the file watcher
				SetupWatcher(new System.IO.FileSystemEventHandler(Config_Changed));
			}
			JobEntity entity = null;
			
			try
			{
				entity = _entities[name] as JobEntity;
			
			}
			catch(Exception){}

			if(entity == null) 
			{
				throw new FactoryException("JobEntity with name '" + name + "' not found. AOMFactory : GetEntity");
			}

			return entity;
		}
		#endregion

		#region Factory Loader
		
		/// <summary>
		/// 
		/// </summary>
		/// <remarks>None</remarks>
		private static void LoadData() 
		{
			ConfigManager config = new ConfigManager("UserConfiguration.config");
			DirectoryPath = config.DirectoryPath;
			FileName = config.FileName;

			ArrayList sourcelist = config.GetAttributeValues("entityAttribute");
			
			foreach(object obj in sourcelist)
			{
				Hashtable hash = (Hashtable)obj;
				string name = Convert.ToString(hash["name"]);
				//the name of this attribute
				if(name == null || name == string.Empty)
					throw new FactoryException("No attribute name specified.");
				
				//get the attribute type
				string strAttributeType = Convert.ToString(hash["attributeType"]);
				if(strAttributeType == null || strAttributeType == string.Empty)
					throw new FactoryException("No Attribute Type specified for attribute " + name);
				//get the type for a strongly typed paramter
				Type attributeType = Type.GetType(strAttributeType);
				if(attributeType == null)
					throw new FactoryException("Could not load class Type for type " + strAttributeType + " for  attribute " + name);
								
				object attributeValue = Convert.ChangeType(hash["attributeValue"],attributeType);
								
				if(_entityAttributes == null)
					_entityAttributes = new Hashtable();
				
				EntityAttribute attribute = new EntityAttribute(name,attributeValue,attributeType);

				if(!_entityAttributes.Contains(name))
					_entityAttributes.Add(name,attribute);
				
			}	
			
			sourcelist = config.GetAttributeValues("entityType");
			
			foreach(object obj in sourcelist)
			{
				Hashtable hash = (Hashtable)obj;
				string name = Convert.ToString(hash["name"]);
				//the name of this attribute
				if(name == null || name == string.Empty)
					throw new FactoryException("No entity name specified.");
				//get the entity type
				string strEntityType = Convert.ToString(hash["type"]);
				if(strEntityType == null || strEntityType == string.Empty)
					throw new FactoryException("No Entity Type specified for [type]");
				//get the type for a strongly typed paramter
				Type entityTypeOf = Type.GetType(strEntityType);
				if(entityTypeOf == null)
					throw new FactoryException("Could not load class Type for type " + strEntityType + " for  entity " + name);
				//get the entity type attributes
				string[] attributes = Convert.ToString(hash["attributes"]).Split(new char[]{','});
				
				if(_entityTypes == null)
					_entityTypes = new Hashtable();
				
				EntityType entityType = (EntityType)Activator.CreateInstance( entityTypeOf,new object[]{name,entityTypeOf});
				
				foreach(string attribute in attributes)
				 if(!entityType.Attributes.Contains((EntityAttribute)_entityAttributes[attribute]))
					entityType.Attributes.Add((EntityAttribute)_entityAttributes[attribute]);

				if(!_entityTypes.Contains(name))
					_entityTypes.Add(name,entityType);
				
			}				
	
			
			sourcelist = config.GetAttributeValues("entity");
			
			foreach(object obj in sourcelist)
			{
				Hashtable hash = (Hashtable)obj;
				string name = Convert.ToString(hash["name"]);
				//the name of this attribute
				if(name == null || name == string.Empty)
					throw new FactoryException("No entity name specified.");
				//get the entity type
				string strEntityClassType = Convert.ToString(hash["type"]);
				if(strEntityClassType == null || strEntityClassType == string.Empty)
					throw new FactoryException("No Entity Class specified for " + strEntityClassType);
				//get the type for a strongly typed paramter
				Type entityClassType = Type.GetType(strEntityClassType);
				if(entityClassType == null)
					throw new FactoryException("Could not load class Type for type " + strEntityClassType + " for  entity " + name);
				//get the entity type
				string strEntityType = Convert.ToString(hash["entityType"]);
				if(strEntityType == null || strEntityType == string.Empty)
					throw new FactoryException("No Entity Type specified for " + strEntityType);
				
				if(_entities == null)
					_entities = new Hashtable();
				
				JobEntity entityClass = 
					(JobEntity)Activator.CreateInstance( 
					entityClassType,new object[]{_entityTypes[strEntityType]});
				
				if(!_entities.Contains(name))
					_entities.Add(name,entityClass);
				
			}		
		}
		#endregion

		#region FileWatcher Members
		/// <summary>
		/// Sets up filewatcher to monitor the config file for changes
		/// </summary>
		/// <remarks>None</remarks>
		/// <param name="method">FileSystemEventHandler as delegate event method</param>
		internal static void SetupWatcher(FileSystemEventHandler method) 
		{
			FileSystemWatcher watcher = new FileSystemWatcher(_directoryPath, _fileName);
			watcher.NotifyFilter = NotifyFilters.LastWrite;
			watcher.Changed += new FileSystemEventHandler(method);
			watcher.EnableRaisingEvents = true;			
		}


		#endregion

		#region FileWatcher Event
		
		/// <summary>
		/// 
		/// </summary>
		/// <remarks>None</remarks>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private static void Config_Changed(object sender, FileSystemEventArgs e) 
		{
			LoadData();			
		}
		#endregion

		
	}
	#endregion

	#region FactoryException
	/// <summary>
	/// 
	/// </summary>
	internal class FactoryException : Exception
	{
		internal FactoryException(string message) : base(message){}
	}
	#endregion
}
