/* *******************************************************
 * This code created and or modified by Chris Lasater
 * http://www.geocities.com/lasaterconsult
 * Please refer all questions or comments to chris.lasater@gmail.com
 * 
 * You can use this code however you like, but please, make sure you give me 
 * credit on any publishings you do from this code. 
 * 
 * No warrenty is implied for any code here.
 * 
 *  ******************************************************* 
 */


using System;
using System.Xml;
using System.IO;
using System.Data;
using System.Reflection;
using System.Collections;
using System.Globalization;
using System.Configuration;

namespace Properties.Configuration
{
	
	/// <summary>
	/// Used to read and update configuration files
	/// </summary>
	/// <remarks>This class uses the DataSet ReadXml function to upload xml</remarks>
	public class ConfigManager 
	{
		#region Private Variables
		// Privates
		private string _filePathName = string.Empty;
		string _directoryPath = string.Empty;
		string _fileName = string.Empty;

		private DataSet _dsDataSet;
		private DataView _dvDataView;
		private string _keyName = "key";
		private string _valueName="value";
		#endregion
		
		#region Constructor and Initiation
		
		
		// Constructor
		/// <summary>
		/// overloaded constructor
		/// Loads file from path and filename.	
		/// -or- 
		/// Loads file from the current assembly path.	
		/// </summary>
		/// <remarks>must supply file name to load</remarks>
		/// <example><code>
		/// ConfigManager config = new ConfigManager("DAL.config");			
		/// connectionString = config.GetValue("application","name","Logger",AppConst.CONNECTION_STRING,"");
		/// </code>
		/// </example>
		/// <param name="fileName">string - file name to load, no path</param>
		public ConfigManager(string fileName) : 
			this (null,fileName,null,null)
		{
				
		}
		// Constructor
		/// <summary>
		/// overloaded constructor
		/// Loads file from path and filename.	
		/// -or- 
		/// Loads file from the current assembly path.	
		/// </summary>
		/// <remarks>must supply file name and path to load</remarks>
		/// <example><code>
		/// ConfigManager config = new ConfigManager("C:/temp","Project.config");			
		/// connectionString = config.GetValue("application","name","Logger",AppConst.CONNECTION_STRING,"");
		/// </code>
		/// </example>
		/// <param name="path">Path where config file resides : "C:/temp"</param>
		/// <param name="fileName">string - file name to load, no path</param>
		public ConfigManager(string path, string fileName) : 
			this (path,fileName,null,null)
		{					
		}
				
		/// <summary>
		/// Loads file from path and filename.	
		/// -or- 
		/// Loads file from the current assembly path.	
		/// </summary>
		/// <remarks> 
		/// Inputs the default key- value attribute names for looking through a
		/// xml file
		/// <code>
		/// &lt;nodeName&gt;
		/// 		&lt;node (keyAttributeName)key="key1" (valueAttributeName)value="value1" ...
		/// </code>	
		/// </remarks>
		/// <example><code>
		/// ConfigManager config = new ConfigManager(true);			
		/// connectionString = config.GetValue("application","name","Logger",AppConst.CONNECTION_STRING,"");
		/// </code>
		/// </example>
		/// <param name="path">the path without the file name</param>
		/// <param name="fileName">string - file name to load, no path</param>
		/// <param name="keyAttributeName">string - changes the 'key' attribute default to look for entered name instead</param>
		/// <param name="valueAttributeName">string - changes the 'value' attribute default to look for entered name instead</param>
		public ConfigManager(
			string path, 
			string fileName, 
			string keyAttributeName, 
			string valueAttributeName)
		{
			_keyName = keyAttributeName;
			_valueName= valueAttributeName;	
			_fileName = fileName;
			_directoryPath = path;

			if(_fileName == null || _fileName == String.Empty)
				throw new ArgumentException("Configuration file name must be provided");

			if(path == null || path == String.Empty)
				_directoryPath = 
						Path.GetDirectoryName(GetValidAssembly().CodeBase.Replace(@"file:///",""));
								
			
			_filePathName = _directoryPath + Path.DirectorySeparatorChar + _fileName;
			if(!_filePathName.EndsWith(".config") && _filePathName.EndsWith(".DLL"))
				_filePathName = 
					_filePathName.Replace(".DLL", ".config");
			if(!_filePathName.EndsWith(".config") && _filePathName.EndsWith(".dll"))
				_filePathName = 
					_filePathName.Replace(".dll", ".config");
			if(!_filePathName.EndsWith(".config") && _filePathName.EndsWith(".EXE"))
				_filePathName = 
					_filePathName.Replace(".EXE", ".config");
			if(!_filePathName.EndsWith(".config") && _filePathName.EndsWith(".exe"))
				_filePathName = 
					_filePathName.Replace(".exe", ".config");
			LoadDataSource();
			 
		}
		/// <summary>
		/// Loads the file into a DataSet
		/// </summary>
		/// <remarks>throws\catches Exception if xml is bad or file does not exist. 
		/// Logs Exception</remarks>
		public void LoadDataSource() 
		{
			
			_dsDataSet = new DataSet();
			try 
			{
				
				_dsDataSet.ReadXml(_filePathName);
				_dvDataView = _dsDataSet.Tables[0].DefaultView;
				_dvDataView.AllowEdit = true;
				_dvDataView.AllowDelete = true;
				_dvDataView.AllowNew = true;
				
			} 
			catch(Exception) 
			{
			
			}

		}
		#endregion

		#region Getter/Setters

		
		/// <summary>
		/// Path with out filename
		/// </summary>
		/// <remarks>None</remarks>
		/// <value>string</value>		
		public string DirectoryPath
		{
			get{return _directoryPath;}
		}
		/// <summary>
		/// File name only
		/// </summary>
		/// <remarks>None</remarks>
		/// <value>string</value>		
		public string FileName
		{
			get{return _fileName;}
		}
		/// <summary>
		/// Returns file path and name
		/// </summary>
		/// <value>string</value>
		/// <remarks>None</remarks>
		public string FilePathName
		{
			get{return _filePathName;}
		}
		/// <summary>
		/// Getter/Setter for 'Key' attribute name
		/// </summary>
		/// <value>string</value>
		/// <remarks>None</remarks>
		public string KeyName 
		{
			get{return _keyName;}
			set{_keyName = value;}
		}
		/// <summary>
		/// Getter/Setter for 'Value' attribute name
		/// </summary>
		/// <value>string</value>
		/// <remarks>None</remarks>
		public string ValueName 
		{
			get{return _valueName;}
			set{_valueName = value;}
		}	
		
		#endregion

		#region Public Methods
		/// <summary>
		/// Returns a single value, first found from xml
		/// </summary>
		/// <remarks>None</remarks>
		/// <example>
		/// <code>ConfigManager config = new ConfigManager(true);			
		/// connectionString = config.GetValue("application","name",	
		/// "Logger",AppConst.CONNECTION_STRING,"");	
		/// xml.......	
		/// &lt;Logger&gt; 			
		/// &lt;application name="....	
		/// &lt;application name="....	
		/// ....	</code>
		/// </example>
		/// <param name="nodeName">the name of the xml node(s) to search through</param>
		/// <param name="keyName">key attribute name to find, if null then finds first node with node name</param>
		/// <param name="keyValue">key attribute value to find, otherwise adds the value to the config file, if null then finds first node with node name</param>
		/// <param name="attributeName">the actual name of the attribute you want the value for</param>
		/// <param name="defaultObject">what to return if there is no value found</param>
		/// <returns>string</returns>
		public string GetValue(string nodeName, string keyName, 
			string keyValue, string attributeName, object defaultObject) 
		{
			
			string strResult=String.Empty;
			if(defaultObject != null) strResult=Convert.ToString(defaultObject);
			DataRowView row;
			
			DataTable table = _dvDataView.DataViewManager.DataSet.Tables[nodeName];
			if(table != null && keyName != null && keyName != String.Empty &&
				keyValue != null && keyValue != String.Empty)
				table.DefaultView.RowFilter=keyName + "='" + keyValue + "'";
			
			if(table != null && table.DefaultView.Count != 0) 
			{
				row = table.DefaultView[0];
				strResult = Convert.ToString(row[attributeName]);
			} 
			else 
			{
				
				AddValue(nodeName, keyName, keyValue, attributeName, defaultObject);
			}
			
			return(strResult);
		}
		
		/// <summary>
		/// returns an array list of hashtable values that contain the attribute values by name
		/// </summary>
		/// <remarks>
		/// If the node is a member of
		/// a collection of named nodes, then call the common node name to get the
		/// collection:
		/// <example>
		/// <code>ConfigManager config = new ConfigManager(true);			
		/// ArrayList values = config.GetAttributeValues("message");	
		/// xml:	
		/// &lt;message name="nameOfCollectionMemember" description="other attributes"&gt;
		/// Some text .. or a node
		/// &lt;/message&gt;
		/// &lt;message name="nameOf2ndCollectionMemember" description="other attributes"&gt;
		/// Some text .. or a node
		/// &lt;/message&gt;
		/// </code>
		/// Returns all the 'message' nodes for the like named collection. 
		/// To retrieve the contents of a node containing text call the name
		/// of the node itself from the Hashtable for that ro:
		/// <code>
		/// ConfigManager config = new ConfigManager(true);			
		/// ArrayList values = config.GetAttributeValues("message");	
		/// foreach(object obj in values)
		/// {
		///		Hashtable hash = (Hashtable)obj;
		///		//get the text inside the node
		///		string message = Convert.ToString(hash["message"]);
		///		string name = Convert.ToString(hash["name"]);
		/// }
		/// 
		/// message variable returns:
		/// 'Some text .. or a node'
		/// </code>
		/// </example>		
		/// </remarks>		
		/// <param name="sectionName">the name of the xml node(s) to search through</param>
		/// <returns>ArrayList</returns>
		public ArrayList GetAttributeValues(string sectionName) 
		{
			
			ArrayList list = new ArrayList();
			try 
			{
				_dvDataView.RowFilter="";	
				DataTable table = _dvDataView.DataViewManager.DataSet.Tables[sectionName];
				if(table != null)
				foreach(DataRow row in table.Rows) 
				{		
					Hashtable attributes = new Hashtable();					
					foreach(DataColumn column in row.Table.Columns)
					{
						string columnName = column.ColumnName;
						attributes.Remove(columnName);
						if(columnName != null && columnName.EndsWith("_Text"))
							columnName = columnName.Replace("_Text","");
						attributes.Add(columnName,row[column]);
					}
					list.Add(attributes);
				}
			} 
			catch (Exception)
			{
				
			}
			return list;
		}

		/// <summary>
		/// adds a value to the xml config
		/// </summary>
		/// <remarks>None</remarks>
		/// <example><code>
		/// ConfigManager config = new ConfigManager(true);			
		/// connectionString = config.AddValue("application","name",	
		/// "Logger",AppConst.CONNECTION_STRING,"some connection string....");	
		/// xml.......	
		/// &lt;Logger&gt; 			
		/// &lt;application name="Logger" connectionString="some connection string...."&gt;	 	
		/// ....	</code>
		/// </example>
		/// <param name="sectionName">the name of the xml node(s) to search through</param>
		/// <param name="keyName">key attribute name to find</param>
		/// <param name="keyValue">key attribute value to find, otherwise adds the value to the config file</param>
		/// <param name="attributeName">the actual name of the attribute you want the value for</param>
		/// <param name="valueObject">the value to write</param>
		/// <returns>bool</returns>
		public bool AddValue(
			string sectionName, 
			string keyName, 
			string keyValue, 
			string attributeName, 
			object valueObject) 
		{
			
			DataRowView row;
			
			try 
			{
				DataTable table = _dvDataView.DataViewManager.DataSet.Tables[sectionName];
				if(table != null && keyName != null && keyName != String.Empty &&
					keyValue != null && keyValue != String.Empty)
					table.DefaultView.RowFilter=keyName + "='" + keyValue + "'";
			
				
				if(_dvDataView.Count==0) 
				{
					row = _dvDataView.AddNew();
					row[keyName] = keyValue;
					row[attributeName] = valueObject;
					row.EndEdit();
					SaveData();
					return true;
				} 
				else 
				{
					return false;
				}
				
			} 
			catch(Exception) 
			{				
				return false;
			}
		}
		
		/// <summary>
		/// removes the row from the xml file
		/// </summary>
		/// <remarks>None</remarks>
		/// <example><code>
		/// ConfigManager config = new ConfigManager(true);			
		/// connectionString = config.RemoveValue("application");	
		/// xml.......	
		/// &lt;Logger&gt; 			
		/// &lt;/Logger&gt;	
		/// ....	</code>
		/// </example>
		/// <param name="keyName">the node name to delete</param>
		/// <returns>bool</returns>
		public bool RemoveValue(string keyName) 
		{
			DataRowView row;
			_dvDataView.RowFilter= _keyName + "='" + keyName + "'";
			
			if(_dvDataView.Count != 0) 
			{
				row = _dvDataView[0];
				row.Delete();
				return true;
			} 
			else 
			{
				return false;
			}
			
		}
		/// <summary>
		/// updates a node attribute
		/// </summary>
		/// <remarks>None</remarks>
		/// <example><code>
		/// ConfigManager config = new ConfigManager(true);			
		/// connectionString = config.UpdateValue("application","name",	
		/// "Logger",AppConst.CONNECTION_STRING,"some connection string....");	
		/// xml.......	
		/// &lt;Logger&gt; 			
		/// &lt;application name="Logger" connectionString="some connection string...."&gt; 		
		/// ....	</code>
		/// </example>
		/// <param name="sectionName">the name of the xml node(s) to search through</param>
		/// <param name="keyName">key attribute name to find</param>
		/// <param name="keyValue">key attribute value to find, otherwise adds the value to the config file</param>
		/// <param name="attributeName">the actual name of the attribute you want the value for</param>
		/// <param name="valueObject">the value to write</param>
		/// <returns></returns>
		public bool UpdateValue(string sectionName, 
			string keyName, 
			string keyValue, 
			string attributeName, 
			object valueObject) 
		{
			
			RemoveValue(keyName);		
			return AddValue(sectionName, keyName, keyValue, attributeName, valueObject);	
		}
		#endregion

		#region Private Methods
		/// <summary>
		/// performs a WriteXml on the dataset
		/// </summary>
		/// <remarks>None</remarks>
		private void SaveData() 
		{
			_dsDataSet.WriteXml(_filePathName);
		}
		/// <summary>
		/// 
		/// </summary>
		/// <remarks>None</remarks>
		/// <returns></returns>
		private DataTable CreateDataTable(string tablename) 
		{
			
			DataTable tbl = new DataTable();
			tbl.TableName = tablename;
			tbl.Columns.Add(_keyName,typeof(string));
			tbl.Columns.Add(_valueName,typeof(string));
			return(tbl);
		}
		

		private Assembly GetValidAssembly()
		{
			Assembly assemblyBase = null;
			if(Assembly.GetEntryAssembly() != null)
			{
				assemblyBase = Assembly.GetEntryAssembly();
			}
			else if(Assembly.GetCallingAssembly () != null)
			{
				assemblyBase = Assembly.GetCallingAssembly ();
			}
			else if(Assembly.GetExecutingAssembly() != null)
			{
				assemblyBase = Assembly.GetExecutingAssembly();
			}
			return assemblyBase;
		}
		#endregion

				
	}
	
}

